import { Test, TestingModule } from '@nestjs/testing';
import { getRepositoryToken } from '@nestjs/typeorm';
import { mockedUsersRepository, userModel } from '../utils/mocks/users.service';
import User from './users.entity';
import { UsersService } from './users.service';

describe('UsersService', () => {
  let service: UsersService;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        UsersService,
        {
          provide: getRepositoryToken(User),
          useValue: mockedUsersRepository,
        },
      ],
    }).compile();

    service = module.get<UsersService>(UsersService);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('when getting a user by discordId', () => {
    beforeEach(() => {
      mockedUsersRepository.findOneBy.mockImplementationOnce(
        ({ discordId }) => {
          if (discordId === userModel.discordId) {
            return userModel;
          }
          return undefined;
        },
      );
    });
    describe('and the user is matched', () => {
      it('should return the user', async () => {
        const fetchedUser = await service.getByDiscordId(userModel.discordId);
        expect(fetchedUser).toEqual(userModel);
      });
    });
    describe('and the user is not matched', () => {
      it('should throw an error', async () => {
        await expect(service.getByDiscordId('unknown')).rejects.toThrow();
      });
    });
  });

  describe('when getting a user by username', () => {
    beforeEach(() => {
      mockedUsersRepository.findOneBy.mockImplementationOnce(({ username }) => {
        if (username === userModel.username) {
          return userModel;
        }
        return undefined;
      });
    });
    describe('and the user is matched', () => {
      it('should return the user', async () => {
        const fetchedUser = await service.getByUserName(userModel.username);
        expect(fetchedUser).toEqual(userModel);
      });
    });
    describe('and the user is not matched', () => {
      it('should throw an error', async () => {
        await expect(service.getByUserName('unknown')).rejects.toThrow();
      });
    });
  });

  describe('when creating a user', () => {
    it('should return the user', async () => {
      mockedUsersRepository.create.mockResolvedValueOnce(userModel);
      const fetchedUser = await service.create(userModel);
      expect(fetchedUser).toEqual(userModel);
    });
  });
});
